/* File: nprofile.c */

/*-----------------------------------------------------------------**
** Copyright (C) The Higher Education Statistics Agency 1994,1995  **
** Authors:                                                        **
** 	Duncan Charlton: DuncanC@hesa.ac.uk                            **
** 	Adrian Crossley: AdrianC@hesa.ac.uk                            **
**-----------------------------------------------------------------*/
	  
#include <stdlib.h>
#include <string.h>
                    
#include "vldtn.h"                   
#include "helper.h"


struct ProfileItem {
  char *psEntry;
  char *psValue;          
  struct ProfileItem *ppiNext;
	};
	

struct ProfileSection {
	char *psSection;
	struct ProfileItem *ppiHead;
	struct ProfileSection *ppsNext;
	};
	
struct ProfileSection *ppsHead=NULL;

	
FILE *pf=NULL;                
  
#ifdef ANSI  
struct ProfileItem *NewProfileItem(const char *lpe,const char *lpv)
#else
struct ProfileItem *NewProfileItem(lpe,lpv)
const char *lpe;
const char *lpv;
#endif
{
	struct ProfileItem *ppiNew;
	                          
	if (!(ppiNew=(struct ProfileItem *) malloc(sizeof(struct ProfileItem))))
		return NULL;
	                          

	ppiNew->psEntry=strdup(lpe);
	ppiNew->psValue=strdup(lpv);
	ppiNew->ppiNext=NULL;
	
	return ppiNew;	
}
          
#ifdef ANSI              
void DeleteProfileItem(struct ProfileItem *ppi)
#else                                          
void DeleteProfileItem(ppi)
struct ProfileItem *ppi;
#endif
{
	if (!ppi)
		return;
		
	if (ppi->psEntry)		
		free(ppi->psEntry);

	if (ppi->psValue)		
		free(ppi->psValue);

}
              
#ifdef ANSI                              
struct ProfileSection *NewProfileSection (const char *psS)
#else                                               
struct ProfileSection *NewProfileSection (psS)
const char *psS;
#endif
{ 
	struct ProfileSection *ppsNew;
	
	if (!(ppsNew=(struct ProfileSection *) malloc(sizeof(struct ProfileSection))))
		return NULL;
		
	ppsNew->psSection=strdup(psS);
	/* printf("Add Section: %s\n",psS);*/
	
	ppsNew->ppiHead=NULL;
	ppsNew->ppsNext=NULL;
			    
	return ppsNew;			    
		
}	
                                          
#ifdef ANSI                                          
void DeleteProfileSection(struct ProfileSection *pps)
#else                                                
void DeleteProfileSection(pps)
struct ProfileSection *pps;
#endif
#
{
	if (!pps)
		return;
		
	if (pps->psSection)
		free(pps->psSection);
		
	free(pps);		
}


#ifdef ANSI
int OpenFile(const char *psFile)
#else                     
int OpenFile(psFile)
const char *psFile;
#endif
{
	/* open file or return default if can't open */
	if((pf=fopen(psFile,"r+"))==NULL)
		{                                  
		/* Can't open the file, assume it doesn;t exist */
		if((pf=fopen(psFile,"a+"))==NULL)		
			return 0;
		}


	return 1;
}
                          
#ifdef ANSI                          
int CloseFile(void)
#else              
int CloseFile()
#endif
{          
	if (pf) 
		fclose (pf);
		
	pf=NULL;		
		
	return 1;
}                                    
              
#ifdef ANSI              
int ListProfile(void)
#else                
int ListProfile()
#endif
{
	struct ProfileSection *pps;
	struct ProfileItem *ppi;
	
	pps=ppsHead;
	
	while (pps) {
		printf(">>%s\n",pps->psSection); 
		ppi=pps->ppiHead;
		while (ppi){       
			printf(">>>%s=%s\n",ppi->psEntry,ppi->psValue); 
			ppi=ppi->ppiNext;
			}
		pps=pps->ppsNext;
		}

	return 1;		
}
                
                
#ifdef ANSI                
struct ProfileSection *FindSection(const char *psSection) 
#else                                               
struct ProfileSection *FindSection(psSection) 
const char *psSection;
#endif
{
	struct ProfileSection *pps;
	
	pps=ppsHead;
	
	while (pps) {
		/* printf(">%s<>%s<\n",psSection,pps->psSection); */
		if (!strcmpi(psSection,pps->psSection)) {
			/* printf("Found!\n"); */
			return pps;                            
			}
		pps=pps->ppsNext;
		}

	return NULL;		
}

#ifdef ANSI
struct ProfileItem *FindEntry(struct ProfileSection *pps,const char *psEntry) 
#else                                                                   
struct ProfileItem *FindEntry(pps,psEntry) 
struct ProfileSection *pps;
const char *psEntry;
#endif
{
	struct ProfileItem *ppi;
	
	ppi=pps->ppiHead;
	
	while (ppi) {
		/* printf(">%s<>%s<\n",psEntry,ppi->psEntry); */
		if (!strcmpi(psEntry,ppi->psEntry)) {
			/* printf("Found!\n"); */
			return ppi;                            
			}
		ppi=ppi->ppiNext;
		}

	return NULL;		
}

#ifdef ANSI
int DeleteProfile(void)
#else                  
int DeleteProfile()
#endif
{
	struct ProfileSection *pps;
	struct ProfileSection *ppsDel;
	struct ProfileItem *ppi;
	struct ProfileItem *ppiDel;
	
	pps=ppsHead;
	
	while (pps) {
		ppsDel=pps;
		/* printf("-%s\n",pps->psSection); */
		ppi=pps->ppiHead;
		while (ppi){       
			/* printf("--%s=%s\n",ppi->psEntry,ppi->psValue); */
			ppiDel=ppi;
			ppi=ppi->ppiNext;
			DeleteProfileItem(ppiDel);
			}
		pps=pps->ppsNext;
		DeleteProfileSection(ppsDel);		
		}
  
  ppsHead=NULL;
  
	return 1;		
}

#ifdef ANSI
int LoadProfile(const char *psFile)
#else                        
int LoadProfile(psFile)
const char *psFile;
#endif
{      
	static char szBuffer[1024];                      
	struct ProfileSection *pps;
	struct ProfileItem *ppi;
	char *pcClose;
	
	OpenFile(psFile);    
	
	
	while (fgets(szBuffer,1024,pf)) {
		if (*szBuffer=='[' && (pcClose=strchr(szBuffer,']'))!=NULL) {
			/* New Section */
			*pcClose=0;
			pps=NewProfileSection(szBuffer+1);
			pps->ppsNext=ppsHead;
			ppsHead=pps;
			}
		else {
			/* Make Sure there's a current section! */
			if (ppsHead) {
				if (strlen(szBuffer))
					*(szBuffer+strlen(szBuffer)-1)=0;
				if ((pcClose=strchr(szBuffer,'='))!=NULL)	{
					/* New Entry */                   
					*pcClose=0;
					ppi=NewProfileItem(szBuffer,pcClose+1);
					ppi->ppiNext=ppsHead->ppiHead;
					ppsHead->ppiHead=ppi;
					}
				}
			}			
		}
	CloseFile();		
	return 1;	
}

#ifdef ANSI
int SaveProfile(const char *psFile)
#else                        
int SaveProfile(psFile)
const char *psFile;
#endif
{
	struct ProfileSection *pps;
	struct ProfileItem *ppi;
	
	OpenFile(psFile);    
	
	pps=ppsHead;
	
	while (pps) {
		fprintf(pf,"[%s]\n",pps->psSection); 
		ppi=pps->ppiHead;
		while (ppi){       
			fprintf(pf,"%s=%s\n",ppi->psEntry,ppi->psValue); 
			ppi=ppi->ppiNext;
			}
		pps=pps->ppsNext;
		fprintf(pf,"\n"); 
		}              
		
	CloseFile();		
	return 1;		
}


#ifdef ANSI          
int FileGetPrivateProfileString(const char *psSection,const char *psEntry,const char *psDefault,char *psReturnBuffer,int cbReturnBuffer,const char * psFilename)
#else
int FileGetPrivateProfileString(psSection,psEntry,psDefault,psReturnBuffer,cbReturnBuffer,psFilename)
const char *psSection;
const char *psEntry;
const char *psDefault;
char *psReturnBuffer;
int cbReturnBuffer;
const char * psFilename;
#endif
{                            
	struct ProfileSection *pps;         
	struct ProfileItem *ppi;         
	LoadProfile(psFilename);	
	  
	if (!(pps=FindSection(psSection))) {
		DeleteProfile();	                
		strncpy(psReturnBuffer,psDefault,cbReturnBuffer-1);
		return strlen(psReturnBuffer);
		}

	if (!(ppi=FindEntry(pps,psEntry))) {
		DeleteProfile();	                
		strncpy(psReturnBuffer,psDefault,cbReturnBuffer-1);
		return strlen(psReturnBuffer);
		}
                              
	strncpy(psReturnBuffer,ppi->psValue,cbReturnBuffer-1);		
	DeleteProfile();	                
	return strlen(psReturnBuffer);
	
}

#ifdef ANSI
int FileWritePrivateProfileString(const char *psSection,const char *psEntry,const char *psValue,const char *psFilename)        
#else
int FileWritePrivateProfileString(psSection,psEntry,psValue,psFilename)        
const char *psSection;
const char *psEntry;
const char *psValue;
const char *psFilename;
#endif
{
	struct ProfileSection *pps;         
	struct ProfileItem *ppi;         
	LoadProfile(psFilename);	
	
	
	if (!(pps=FindSection(psSection))) {
		/* Can't find the section, better make one */
		pps=NewProfileSection(psSection);
		pps->ppsNext=ppsHead;
		ppsHead=pps;
		}

	if (!(ppi=FindEntry(pps,psEntry))) {
		/* Can't find the Entry, better make one */
		ppi=NewProfileItem(psEntry,psValue);
		ppi->ppiNext=pps->ppiHead;
		pps->ppiHead=ppi;
		}


	if (ppi->psValue) {
		free(ppi->psValue);
		ppi->psValue=strdup(psValue);
		}
    
	SaveProfile(psFilename);    
  DeleteProfile();	                
	return 1;
}


